     /**
      * \brief List Utilities
      *
      * This module contains several procedures which uses the linked
      * list procedures for storing simple data.
      *
      * \author Mihael Schmidt
      * \date   23.02.2008
      *
      * \rev 15.03.2008 Mihael Schmidt
      *      added procedure lutil_listObjects
      */


      *------------------------------------------------------------------------
      *
      * Copyright (c) 2007-2008 Mihael Schmidt
      * All rights reserved.
      *
      * This file is part of the LUTIL service program.
      *
      * LUTIL is free software: you can redistribute it and/or modify it under
      * the terms of the GNU Lesser General Public License as published by
      * the Free Software Foundation, either version 3 of the License, or
      * any later version.
      *
      * LUTIL is distributed in the hope that it will be useful,
      * but WITHOUT ANY WARRANTY; without even the implied warranty of
      * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
      * GNU Lesser General Public License for more details.
      *
      * You should have received a copy of the GNU Lesser General Public
      * License along with LUTIL.  If not, see http://www.gnu.org/licenses/.
      *
      *------------------------------------------------------------------------


     HNOMAIN
     HBNDDIR('QC2LE')
     HOPTIMIZE(*NONE)
     HDEBUG(*YES)
     HOPTION(*SRCSTMT)
     HCOPYRIGHT('Copyright (c) 2007-2008 Mihael Schmidt. All rights reserved.')


      *------------------------------------------------------------------------
      * Prototypen
      *------------------------------------------------------------------------
      /copy LUTIL_H
      /copy LLIST_H
      /copy QUSLMBR_H
      /copy QUSLRCD_H
      /copy QUSLOBJ_H
      /copy USRSPC_H
      /copy API_ERR_H


     /**
      * \brief List file member names
      *
      * All member names of a file are placed into a linked list.
      * This list is returned to the caller. On any error *null is
      * returned.
      *
      * \author Mihael Schmidt
      * \date   23.02.2008
      *
      * \param Library name
      * \param File name
      *
      * \return pointer to list or *null if any error occurs
      */
     P lutil_listFileMembers...
     P                 B                   export
     D                 PI              *
     D   library                     10A   const
     D   filename                    10A   const
      *
     D list            S               *
     D usPtr           S               *
     D usHeader        DS                  likeds(userspace_gen) based(usPtr)
     D usName          S             10A   inz('LUTIL1')
     D usLib           S             10A   inz('QTEMP')
     D dataPtr         S               *
     D usData          S             10A   based(dataPtr)
     D i               S             10I 0
      /free
       monitor;

         createUserSpace(usName + usLib : *blank : 4096 : X'00' : '*ALL' :
                         'List Util UserSpace' : '*YES' : apiErrDs );
         if (apiErrCode <> *blank);
           return *null;
         endif;

         // change user space attributes to autoextend
         userspace_attr.size = 1;       // 1=attribute will be changed
         userspace_attr.key = 3;        // 3=extensibility attribute
         userspace_attr.dataLength = 1;
         userspace_attr.data = '1';     // user space is extensible
         changeUserspaceAttr(usLib : usName + usLib : userspace_attr:apiErrDs);
         if (apiErrCode <> *blank);
           return *null;
         endif;

         retrieveUserspacePtr(usName + usLib : usPtr : apiErrDs);
         if (apiErrCode <> *blank);
           return *null;
         endif;

         listFileMembers( usName + usLib : 'MBRL0100' : filename + library :
                          '*ALL' : '1' : apiErrDs);
         if (apiErrCode <> *blank);
           return *null;
         endif;

         list = list_create();

         // iterate through retrieved members
         for i = 0 to usHeader.nmbrEntries - 1;
           dataPtr = usPtr + usHeader.offsetList + (i * usHeader.entrySize);
           list_add(list : %addr(usData) : %len(%trim(usData)));
         endfor;

         deleteUserspace(usName + usLib : apiErrDs);

         on-error *all;
           if (list <> *null);
             list_dispose(list);
           endif;
           return *null;

       endmon;

       return list;
      /end-free
     P                 E

     /**
      * \brief List record format names
      *
      * All record format names of a file are placed into a linked list.
      * This list is returned to the caller. On any error *null is
      * returned.
      *
      * \author Mihael Schmidt
      * \date   25.02.2008
      *
      * \param Library name
      * \param File name
      *
      * \return Pointer to list or *null if any error occured
      */
     P lutil_listRecordFormats...
     P                 B                   export
     D                 PI              *
     D   library                     10A   const
     D   filename                    10A   const
      *
     D list            S               *
     D usPtr           S               *
     D usHeader        DS                  likeds(userspace_gen) based(usPtr)
     D usName          S             10A   inz('LUTIL2')
     D usLib           S             10A   inz('QTEMP')
     D dataPtr         S               *
     D usData          S             10A   based(dataPtr)
     D i               S             10I 0
      /free
       monitor;

         createUserSpace(usName + usLib : *blank : 4096 : X'00' : '*ALL' :
                         'List Util UserSpace' : '*YES' : apiErrDs );
         if (apiErrCode <> *blank);
           return *null;
         endif;

         // change user space attributes to autoextend
         userspace_attr.size = 1;       // 1=attribute will be changed
         userspace_attr.key = 3;        // 3=extensibility attribute
         userspace_attr.dataLength = 1;
         userspace_attr.data = '1';     // user space is extensible
         changeUserspaceAttr(usLib : usName + usLib : userspace_attr:apiErrDs);
         if (apiErrCode <> *blank);
           return *null;
         endif;

         retrieveUserspacePtr(usName + usLib : usPtr : apiErrDs);
         if (apiErrCode <> *blank);
           return *null;
         endif;

         listRecordFormats( usName + usLib : 'RCDL0100' : filename + library :
                          '1' : apiErrDs);
         if (apiErrCode <> *blank);
           return *null;
         endif;

         list = list_create();

         // iterate through retrieved members
         for i = 0 to usHeader.nmbrEntries - 1;
           dataPtr = usPtr + usHeader.offsetList + (i * usHeader.entrySize);
           list_add(list : %addr(usData) : %len(%trim(usData)));
         endfor;

         deleteUserspace(usName + usLib : apiErrDs);

         on-error *all;
           if (list <> *null);
             list_dispose(list);
           endif;
           return *null;

       endmon;

       return list;
      /end-free
     P                 E


     /**
      * \brief List objects
      *
      * All object names are placed into a linked list.
      * This list is returned to the caller. On any error *null is
      * returned.
      *
      * <br><br>
      *
      * The entries in the list have the following format:
      * <ul>
      *   <li>10A Library name</li>
      *   <li>10A Object name</li>
      *   <li>10A Object type</li>
      * </ul>
      *
      * <br><br>
      * As this procedure utilizes the QUSLOBJ i5/OS API the parameters
      * here accept the same names and generic names for library, name and
      * object type.
      *
      * \param Library
      * \param Object name (default: *all)
      * \param Object type (default: *all)
      */
     P lutil_listObjects...
     P                 B                   export
     D                 PI              *
     D   pLibrary                    10A   const
     D   pObject                     10A   const options(*omit : *nopass)
     D   pType                       10A   const options(*nopass)
      *
     D lib             S             10A   inz('*ALL')
     D obj             S             10A   inz('*ALL')
     D type            S             10A   inz('*ALL')
     D usHeader        DS                  likeds(userspace_gen) based(usPtr)
     D usName          S             10A   inz('LUTIL3')
     D usLib           S             10A   inz('QTEMP')
     D usPtr           S               *
      *
     D ptr             S               *
     D i               S             10I 0
     D list            S               *   inz(*null)
      /free
       lib = pLibrary;

       if (%parms() >= 2 and %addr(pObject) <> *null);
         obj = pObject;
       endif;

       if (%parms() = 3);
         type = pType;
       endif;

       // userspace erstellen
       createUserspace(usName + usLib :
                       'DATA' :
                       4096 :
                       X'00' :
                       '*ALL' :
                       %trimr(usName) + ' UserSpace' :
                       '*YES' :
                       apiErrDs );

       userspace_attr.size = 1;   // 1=attribute will be changed
       userspace_attr.key = 3;    // 3=extensibility attribute
       userspace_attr.dataLength = 1;
       userspace_attr.data = '1'; // user space is extensible
       changeUserspaceAttr(usLib : usName + usLib :
                           userspace_attr : apiErrDs);

       retrieveUserSpacePtr(usName + usLib : usPtr : apiErrDs);

       listObjects( usName + usLib :
                    'OBJL0100' :
                    obj + lib :
                    type :
                    apiErrDs );
       if (apiErrCode = *blank);

         list = list_create();

         for i = 0 to usHeader.nmbrEntries - 1;
           ptr = usPtr + usHeader.offsetList + (i * usHeader.entrySize);
           list_add(list : ptr : 30);
         endfor;
       endif;

       // delete userspace
       deleteUserspace(usName + usLib : apiErrDs);

       return list;
      /end-free
     P                 E
